// checkarg.ch
// Copyright (C) 1994 All rights reserved. Paul Long

// With the functions in checkarg.prg, these macros automatically perform type
// checking on the arguments passed in function and procedure calls based on
// the modified Hungarian notation of the parameters.  Include this file at
// the top of the source files where you want to perform type checking then
// compile them with
//    the /dDEBUG option or
//    the /dCHECKARG option if you don't want to use /dDEBUG because it would
//       turn on other debug code.
// If you subsequently don't want type checking, you can leave the inclusions
// of this file in the source files and just turn the macro expansion off by
//    getting rid of the /dDEBUG option or
//    specifying the /dNOCHECKARG option if you want to keep the /dDEBUG option
//       because you want other debug code turned on.
//
// NOTE: Internal UDCs start with CAx.  Do not use them outside this
// header file.


// Enable checking if DEBUG defined.  CHECKARG or NOCHECKARG override.
#ifdef DEBUG
#ifndef NOCHECKARG
#define CHECKARG
#endif
#endif

#ifdef CHECKARG

// Check parameters one at a time then update module status in CAEnd()
// via pass-by-reference operator.
#xtranslate CAxCKxPMS( [ <id1> ] [ , <idN> ] ) =>  ;
      STATIC xCAStat;                              ;
      LOCAL xCADmy := (CABeg(xCAStat),             ;
            [ CACkPm( <"id1">, <id1> ), ]          ;
            [ CACkPm( <"idN">, <idN> ), ]          ;
            CAEnd(pcount(), @xCAStat))

// Prototypes for function definitions with and without an argument list.
#xtranslate CAxFUNC <name> ( [ <id1> ] [ , <idN> ] ) => ;
      FUNCTI <name>( [<id1> ] [, <idN>]); CAxCKxPMS( [ <id1> ] [, <idN> ] )
#xtranslate CAxFUNCxWO <name> => FUNCTI <name>; CAxCKxPMS()

// Prototypes for procedure definitions with and without an argument list.
#xtranslate CAxPROC <name> ( [ <id1> ] [ , <idN> ] ) => ;
      PROCEDU <name>( [<id1> ] [, <idN>]); CAxCKxPMS( [ <id1> ] [, <idN> ] )
#xtranslate CAxPROCxWO <name> => PROCEDU <name>; CAxCKxPMS()

// Functions and procedures that are _not_ static.
#xcommand FUNC <name> => CAxFUNCxWO <name>
#xcommand FUNCTION <name> => CAxFUNCxWO <name>
#xcommand FUNC <name> ( [ <id1> ] [, <idN> ] ) => ;
      CAxFUNC <name> ( [ <id1> ] [ , <idN> ] )
#xcommand FUNCTION <name> ( [ <id1> ] [, <idN> ] ) => ;
      CAxFUNC <name> ( [ <id1> ] [ , <idN> ] )
#xcommand PROC <name> => CAxPROCxWO <name>
#xcommand PROCEDURE <name> => CAxPROCxWO <name>
#xcommand PROC <name> ( [ <id1> ] [, <idN> ] ) => ;
      CAxPROC <name> ( [ <id1> ] [ , <idN> ] )
#xcommand PROCEDURE <name> ( [ <id1> ] [, <idN> ] ) => ;
      CAxPROC <name> ( [ <id1> ] [ , <idN> ] )

// Functions and procedures that are static.
#xcommand STATIC <mod: FUNC, FUNCTION> <name> => STATIC CAxFUNCxWO <name>
#xcommand STATIC <mod: FUNC, FUNCTION> <name> ( [ <id1> ] [, <idN> ] ) => ;
      STATIC CAxFUNC <name> ( [ <id1> ] [ , <idN> ] )
#xcommand STATIC <mod: PROC, PROCEDURE> <name> => STATIC CAxPROCxWO <name>
#xcommand STATIC <mod: PROC, PROCEDURE> <name> ( [ <id1> ] [, <idN> ] ) => ;
      STATIC CAxPROC <name> ( [ <id1> ] [ , <idN> ] )

#endif
