/*
 *
 *  PROGRAM  : fileasso.c
 *
 *  PURPOSE  : Show example of opening mutilple documents via file association
 *				through single instance.
 *
 *  FUNCTIONS:
 *
 *		WinMain() - Calls the initialization function
 *		            and processes message loop
 *
 *		MDIFrameWndProc() - Window function for the "frame"
 *		            window, which controls the menu
 *		            and contains the MDI client window  
 *
 *		SkipWhiteSpace() - removes any spaces from a string.
 *
 *		ParseExecuteString() - compares the string with the existng command,
 *					extracts the  command, and parameters passed with it. 
 *
 *		MDIChildWndProc()- Window function for the individual
 *		            child windows
 *
 *		CloseAllChildren - Destroys all MDI child windows.
 *
 *		CommandHandler() - Processes the "frame" window's
 *		            WM_COMMAND messages.
 *
 *		ConnectServer() - Initializes and registers the ddeml server 
 *
 *		DisconnectServer() - Uninitializes the ddeml server 
 *
 *		DdeServerCallback() - Server callback function, 
 *					This function is called from DDEML when the server 
 * 					receives the execute command, performs accordingly.
 *
 * Copyright 1991 Microsoft Corporation. All rights reserved.
 */


/*--------------------  #includes  -------------------------------*/

#include "fileasso.h"


/*--------------------  global variables  -------------------------*/

// global variables used in this module or among more than one module

HANDLE hInst;                   // Program instance handle
HWND hwndFrame     = NULL;      // Handle to main window
HWND hwndMDIClient = NULL;      // Handle to MDI client
LONG styleDefault  = 0;         // Default style bits for child windows 

/*---------------------  function prototypes  -----------------------*/

// Forward declarations of helper functions in this module

VOID NEAR PASCAL CommandHandler  (HWND,WORD);
VOID NEAR PASCAL CloseAllChildren(VOID);


/*---------------  MDIFrameWndProc()  -------------------------------*/
/*
 *
 *  FUNCTION   : MDIFrameWndProc (hwnd, msg, wParam, lParam )
 *
 *  PURPOSE    : The window function for the "frame" window, which controls
 * 			 the menu and encompasses all the MDI child windows. Does
 * 			 the major part of the message processing. Specifically, in
 * 			 response to:
 *
 * 			 WM_CREATE : Creates and displays the "frame".
 *
 * 			 WM_COMMAND: Passes control to a command-handling function.
 *
 * 			 WM_CLOSE  : Quits the app.
 *
 * 			 WM_DESTROY: Destroys frame window and quits app.
 *
 *  NOTE: If cases are added for WM_MENUCHAR, WM_NEXTMENU, WM_SETFOCUS,
 *        and WM_SIZE, note that these messages should be passed on
 *        to DefFrameProc even if we handle them.  See the SDK Reference
 *        entry for DefFrameProc
 */


LONG FAR PASCAL __export MDIFrameWndProc ( hwnd, msg, wParam, lParam )

register HWND hwnd;
UINT	      msg;
register WPARAM wParam;
LPARAM		lParam;

{
	switch (msg)
		{
		case WM_CREATE:
			{
			CLIENTCREATESTRUCT ccs;
				
			//start the ddeml server for this application
			ConnectServer();
			// Find window menu where children will be listed
			ccs.hWindowMenu  = GetSubMenu (GetMenu(hwnd),WINDOWMENU);
			ccs.idFirstChild = IDM_WINDOWCHILD;

			// Create the MDI client
			hwndMDIClient = CreateWindow ("mdiclient",
					                NULL,
					                WS_CHILD | WS_CLIPCHILDREN,
					                0,
					                0,
					                0,
					                0,
					                hwnd,
					                0,
					                hInst,
					                (LPSTR)&ccs);
			ShowWindow (hwndMDIClient,SW_SHOW);
			break;
			}

		case WM_COMMAND:
			// Direct all menu selection or accelerator commands to
			// the CommandHandler function
			CommandHandler (hwnd,wParam);
			break;

		case WM_CLOSE:
			DisconnectServer();
			DestroyWindow (hwnd);
			break;

		case WM_DESTROY:
			PostQuitMessage (0);
			break;

		default:
			// use DefFrameProc() instead of DefWindowProc(), since there
			// are things that have to be handled differently because of MDI

			return DefFrameProc (hwnd,hwndMDIClient,msg,wParam,lParam);
		}
	return 0;
}


							
#define NUM_COMMANDS 2
char *szMyTokens[] = { "OPEN(",
					   "TEST("};

/*---------------  SkipWhiteSpace()  -------------------------------*/
/*
 *
 *  FUNCTION   : SkipWhiteSpace (string )
 *
 *  PURPOSE    : The function removes blank spaces from a given string.
 *
 *
 */							
 
 void SkipWhiteSpace(char *string)
 {
  char *newString;
  int i,j;
  // allocate a temporary string.
  newString = malloc(sizeof(char)*strlen(string));	
  i = j = 0;                
  // traverse throught the string.
  while(string[i] != '\0')
  {                            
		//skip character if it is blank
		if(string[i] != ' ')
			newString[j++] = string[i];
		i++;
  }       
  // copy the new string 
  strcpy(string,newString); 
  // free the temprory string.  
  free(newString);
}
 
/*---------------  ParseExecuteString()  -------------------------------*/
/*
 *
 *  FUNCTION   :ParseExecuteString (char *pszExecuteStr, int *commandCode,
 *									char * pszParams, int *cParamLen )
 *
 *  PURPOSE    :compares the string with the existng command,
 * 				extracts the  command, and parameters passed with it. 
 *  Limitation: 
 *  		a)execute command has to be of type	"[<action> <param> )]"
 *  		b)list of parameters cannot contain ")".
 */							
BOOL ParseExecuteString(char *pszExecuteStr, int *commandCode, 
						char * pszParams, int *cParamLen)
{     
	int i = 0, tokenLen;
	char *pszExecute;
	int iParamSize;
	            
	//initialize the command to invalid.	            
	*commandCode = -1;
	pszExecute = pszExecuteStr;
	                                    
	//check if the execute string is valid		                                    
		if(pszExecute == NULL)
			return FALSE;
	// remove any blank spaces from the execute string. 		
		SkipWhiteSpace(pszExecute);
		//check if the very first character is [
		if(*pszExecute != '[' )
			return FALSE;                        
	pszExecute++;   		                          
	// iterate thru the list of commands and break when the first match occur.
	for(i=0; i< NUM_COMMANDS ;i++)  
	{
		tokenLen = strlen(szMyTokens[i]);                                         
		
		// compare the commands case in sensitive.
		if(_strnicmp(szMyTokens[i],pszExecute,tokenLen) ==0)
		{
		*commandCode = i;
		pszExecute+= tokenLen;
		break;    
		}                     
	}               
	// code -1, means the string did not have valid command
	if(*commandCode == -1)
		return FALSE;                   
		
	//to extract the parameters, assumptions is there is a ) and it is not part 
	//of parameters list.   	
	iParamSize = strcspn(pszExecute, ")");
	if(iParamSize < 0)
		return FALSE; 
	if(iParamSize > 0)
	{
	   if(*cParamLen < iParamSize + 1)
	   	iParamSize = *cParamLen - 1;           
	   strncpy(pszParams,pszExecute,iParamSize);
	   pszParams[iParamSize] = '\0';
	}   
	// skip the parameters and the )
	pszExecute+= iParamSize + 1;	  
	// check if the next one is ]. just for syntax sake.
	if(pszExecute[0] != ']')
		return FALSE;

	return TRUE;   
}




/*--------------------MDIChildWndProc  ----------------------*/
/*
 *
 *  FUNCTION: MDIChildWndProc ( hwnd, msg, wParam, lParam )
 *
 *  NOTE: If cases are added for WM_CHILDACTIVATE, WM_GETMINMAXINFO,
 *        WM_MENUCHAR, WM_MOVE, WM_NEXTMENU, WM_SETFOCUS, WM_SIZE,
 *        or WM_SYSCOMMAND, these messages should be passed on
 *        to DefMDIChildProc even if we handle them. See the SDK
 *        Reference entry for DefMDIChildProc
 *
 */


LONG FAR PASCAL __export MDIChildWndProc ( hwnd, msg, wParam, lParam )
register HWND hwnd;
UINT	      msg;
register WPARAM wParam;
LPARAM		lParam;

{
	// Since this is a generic MDI app, the children don't handle
	// and messages in non-default ways
	return DefMDIChildProc (hwnd, msg, wParam, lParam);
}



/*-----------------  CommandHandler  -------------------------------*/
/*
 *
 *  FUNCTION   : CommandHandler ()
 *
 *  PURPOSE    : Processes all "frame" WM_COMMAND messages.
 *
 */

VOID NEAR PASCAL CommandHandler ( hwnd, wParam )
register HWND hwnd;
register WORD wParam;

{
	switch (wParam)
		{
		case IDM_FILENEW:
			// Make an empty MDI child window
			MakeNewChild (NULL);
			break;

		case IDM_FILEEXIT:
			// Close application
			SendMessage (hwnd, WM_CLOSE, 0, 0L);
			break;

		case IDM_WINDOWTILE:
			// Tile MDI windows
			SendMessage (hwndMDIClient, WM_MDITILE, 0, 0L);
			break;

		case IDM_WINDOWCASCADE:
			// Cascade MDI windows
			SendMessage (hwndMDIClient, WM_MDICASCADE, 0, 0L);
			break;

		case IDM_WINDOWICONS:
			// Auto - arrange MDI icons
			SendMessage (hwndMDIClient, WM_MDIICONARRANGE, 0, 0L);
			break;

		case IDM_WINDOWCLOSEALL:
			CloseAllChildren();
			break;

		default: 
			// This is essential, since there are frame WM_COMMANDS generated
			// by the MDI system for activating child windows via the
			// window menu.
			DefFrameProc(hwnd, hwndMDIClient, WM_COMMAND, wParam, 0L);
		}
}

/*-------------------  CloseAllChildren  -----------------------------*/
/*
 *
 *  FUNCTION   : CloseAllChildren ()
 *
 *  PURPOSE    : Destroys all MDI child windows.
 *
 */


VOID NEAR PASCAL CloseAllChildren (VOID)
{
	register HWND hwndT;

	// As long as the MDI client has a child, destroy it
	while (hwndT = GetWindow (hwndMDIClient, GW_CHILD))
	{
		// Skip the icon title windows
		while (hwndT && GetWindow (hwndT, GW_OWNER))
			hwndT = GetWindow (hwndT, GW_HWNDNEXT);
		if (hwndT)
			SendMessage (hwndMDIClient, WM_MDIDESTROY, (WORD)hwndT, 0L);
		else
			break;
	}
}



/*-----------------------  WinMain  ----------------------------------*/
/*
 *
 *  FUNCTION   : WinMain(HANDLE, HANDLE, LPSTR, int)
 *
 *  PURPOSE    : Creates the "frame" window, does some initialization and
 *     enters the message loop.
 *
 */


int NEAR PASCAL WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)

HANDLE hInstance;
HANDLE hPrevInstance;
LPSTR  lpszCmdLine;
int    nCmdShow;
{
	MSG msg;

	hInst = hInstance;

	// If this is the first instance of the app. register window classes
	if (!hPrevInstance)
	   if (!InitializeApplication ())
		 return 0;

	// Create the frame and do other initialization
	if (!InitializeInstance (lpszCmdLine, nCmdShow))
	   return 0;

	// Enter main message loop
	while (GetMessage (&msg, NULL, 0, 0))
	{
		// If a keyboard message is for the MDI, let the MDI client
		// take care of it.  Otherwise, just handle the message as usual
		if ( !TranslateMDISysAccel (hwndMDIClient, &msg))
		{
			TranslateMessage (&msg);
			DispatchMessage (&msg);
		}
	}
	return 0;
}
 
 
/*---------------  ConnectServer()  -------------------------------*/
/*
 *
 *  FUNCTION   : ConnectServer ( )
 *
 *  PURPOSE    : Initializes and registers the server.
 *					This server is used in this application to 
 *					communciate with file association  
 *				   serverName is MyServer
 *				   topicName is System
 */							
// server code for the file association 
BOOL ConnectServer()
{
	UINT nDdeRtnStatus;
	DWORD uRes = 0;

	// Initialize the DDE Instance ID to zero for the call to DDEInitialize
	m_idDdeServerInst = 0;

	// Register this application with the DDE Management Library.
	nDdeRtnStatus = DdeInitialize(&m_idDdeServerInst, 
												(PFNCALLBACK)&DdeServerCallback, 
	                                 APPCMD_FILTERINITS, uRes);
	if(DMLERR_NO_ERROR != nDdeRtnStatus)
	{
		OutputDebugString("Unable to Register with DDEML Library");
		return FALSE;
	}
	m_hszServerName = DdeCreateStringHandle(m_idDdeServerInst, "MyServer", 
														CP_WINANSI);
	m_hszTopic = DdeCreateStringHandle(m_idDdeServerInst, "System", CP_WINANSI);    
	// register the server Name 
	DdeNameService(m_idDdeServerInst, m_hszServerName, NULL, DNS_REGISTER);
	return TRUE;
}

/*---------------  DisconnectServer()  -------------------------------*/
/*
 *
 *  FUNCTION   : DisconnectServer ( )
 *
 *  PURPOSE    : Uninitializes the server from ddeml.
 *
 */							

BOOL DisconnectServer()
{              
	// free the server name string handle
	if (m_hszServerName)
	{
		if (!DdeFreeStringHandle(m_idDdeServerInst,m_hszServerName))
		{
			OutputDebugString("Unable to free Dde item string handle\n");
			return FALSE;
		}                  
	}  
	// free the topic string handle
	if (!DdeFreeStringHandle(m_idDdeServerInst,m_hszTopic))
	{
		OutputDebugString("Unable to free Dde item string handle\n");
		return FALSE;
	}                  
	// uninitialize the instance of the server.
	DdeUninitialize(m_idDdeServerInst); 
	m_idDdeServerInst = 0;
  return TRUE;
}



/*---------------  DdeServerCallback()  -------------------------------*/
/*
 *
 *  FUNCTION   : DdeServerCallback ( )
 *
 *  PURPOSE    :  Server callback function, 
 *								This function is called from DDEML when the server 
 *								receives the execute command, performs accordingly.
 */							

HDDEDATA CALLBACK _export DdeServerCallback(UINT uType, UINT uFmt,HCONV hConv, 
														HSZ hsz1, HSZ hsz2,	HDDEDATA hData, 
														DWORD dwData1, DWORD dwData2)
{
	switch (uType)
	{     
		case XTYP_DISCONNECT:
			return(0);
		case	XTYP_CONNECT: 
			// if a client request connection withmy server Name and Topic
			// grant connection.
			if(m_hszServerName == hsz2)
				if(hsz1 == m_hszTopic)
			 		return (HDDEDATA)TRUE;
			return (0);
		case XTYP_EXECUTE :
			{
			char szExecute[80], pszExec[80];
			int iCommand,iParamSize;        
			iParamSize = sizeof(pszExec);
			// reterives the execution string            
			DdeGetData(hData, (LPBYTE)&szExecute, sizeof(szExecute),0L); 
			OutputDebugString((LPSTR)szExecute);                         
			// parse the string for the command and its parameters.
			if(ParseExecuteString(szExecute,&iCommand,pszExec,&iParamSize))
				switch(iCommand)
				{
					case 0:   // execute command Open
						MakeNewChild((LPSTR)pszExec);              
						break;
					case 1:   // execute command Test              
						// this is like when an application is not running, the 
						// application would be launched and sends this execution.
						// like print and quit the application. This doesn't make 
						// sense for Open.
						MakeNewChild((LPSTR)pszExec);
						PostQuitMessage(0);
					default:
						break;
				}	
			 else
			   OutputDebugString("Invalid Execute Command");

			}
			break;
		case XTYP_ERROR:
			OutputDebugString("DDEML Error Encountered in CallBack\n");
				return (0);
		default:  
				break;
	}
	return 0;
}
 