/*
 * File......: SHUFFLE.PRG
 * Author....: Robert S. Tucker
 * BBS.......: ESP Wild!
 * Net/Node..:
 * User Name.: Robert S. Tucker
 * Date......: $Date$
 * Revision..: $Revision$
 * Log file..: $Logfile$
 *
 * This is an original work by Tucker and is placed in the public
 * domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log$
 *
 */

/*  $DOC$
 *  $FUNCNAME$
 *      SHUFFLE()
 *  $CATEGORY$
 *      Screen Saver
 *  $ONELINER$
 *      Shuffle screen saver.
 *  $SYNTAX$
 *      SHUFFLE([<nDelay>][,nPause]) --> NIL
 *  $ARGUMENTS$
 *      <nDelay> is an optional numeric parameter that is the delay to
 *      use between moves. This is in 10ths of a second. Default=.05
 		<nPause> is optional for speed for block redraws during moves.
 		Default = .08
 *  $RETURNS$
 *      Nothing.
 *  $DESCRIPTION$
 *      Shuffle() is a screen saver function.
 *  $EXAMPLES$
 *      // Call the screen saver with a delay of 3/10 of a second.
 *
 *      Shuffle(3)
 *  $END$
 */
static seed

// default all parameters to a specific value if not passed to function
#xcommand DEFAULT <param> TO <value> [, <param2> TO <value2> ] => ;
          IF <param> == NIL ; <param> := <value> ; END ;
          [; IF <param2> == NIL ; <param2> := <value2> ; END ]
#xcommand PAUSE(<x>,<var>) => ;
		  IF INKEY(<x>) # 0;<var> := .f.;END
#Define FRAC(x)  (x-INT(x))

FUNCTION SHUFFLE(nDelay,nPause)

local cBack := savescreen(0,0,maxrow(),maxcol())
local cArray := ARRAY(10,5), i, j := 1, r:=5, c:=8
local nx,ny, top,left,bottom,right
local nDir, nrow:=row(), ncol:=col(), hPause
local bQuit, cSc, anc_cu, navail, anext, nnext, lNotQuit := .t.

default nDelay to .05
default nPause to .08
default hPause To nPause/2

cSc:=savescreen(0,0,maxrow(),maxcol())
anc_cu:=setcursor(0)

setcolor('w/n')
bQuit := {|nDelay| inkey(nDelay) != 0 }

// Store Screen into array.  Only written for 80x25
for i:=1 TO 10
  for j := 1 TO 5
  	 cArray[i][j] := savescreen((j-1)*r,(i-1)*c,(r*j)-1,(c*i)-1)
  next j
next i

// Select one at random, but not on the outside
seed:=FRAC(seconds()/1000)

// starting block
nx := Nurd(8)+2
ny := Nurd(3)+2
nDir := Nurd(4) + 1

// Main loop
do while lNotQuit .and. !eval(bQuit,nDelay)
	   top    := (ny-1)*r
	   left   := (nx-1)*c
	   bottom := (ny*r)-1
	   right  := (nx*c)-1
	   DO CASE
	    CASE nDir == 1  // Up
			FOR i := 1 TO r
			 	 restscreen(top-i,left,bottom-i,right,cArray[nx,ny])
			 	 scroll(bottom-(i-1),left,bottom,right)
			 	 PAUSE(nPause,lNotQuit)
			NEXT
	  	     // Store the moved square into the array slot
	   		 cArray[nx,ny-1] := savescreen(top-r,left,bottom-r,right)
	    CASE nDir == 2  // Down
			FOR i := 1 TO r
			 	 restscreen(top+i,left,bottom+i,right, cArray[nx,ny])
			 	 scroll(top,left,top+(i-1),right)
			 	 PAUSE(nPause,lNotQuit)
			NEXT
	  	     // Store the moved square into the array slot
	   		 cArray[nx,ny+1] := savescreen(top+r,left,bottom+r,right)
	    CASE nDir == 3  // Left
			FOR i := 1 TO c
			 	 restscreen(top,left-i,bottom,right-i,cArray[nx,ny])
			 	 scroll(top,right-(i-1),bottom,right)
			 	 PAUSE(hPause,lNotQuit)
			NEXT
	  	     // Store the moved square into the array slot
	   		 cArray[nx-1,ny] := savescreen(top,left-c,bottom,right-c)
	    CASE nDir == 4  // Right
			FOR i := 1 TO c
			 	 restscreen(top,left+i,bottom,right+i,cArray[nx,ny])
			 	 scroll(top,left,bottom,left+(i-1))
			 	 PAUSE(hPause,lNotQuit)
			NEXT
	  	     // Store the moved square into the array slot
	   		 cArray[nx+1,ny] := savescreen(top,left+c,bottom,right+c)
	   ENDCASE
	   // Next available squares for moving
	   DO CASE
	   CASE nx == 1 .and. ny == 1  // Can only choose 2!
	       	aNext := {{2,1,3}, ; // Right, can only go left
		   		     {1,2,1}}   // below, can only go up
		   		     nAvail := 2
	   CASE nx == 1 .and. ny == 5  // Can only choose 2!
	       	aNext := {{2,5,3}, ; // Right, can only go left
		   		     {1,4,2}}   // below, can only go Down
		   		     nAvail := 2
	   CASE nx == 10 .and. ny == 1  // Can only choose 2!
	       	aNext := {{9,1,4}, ; // Right, can only go right
		   		     {9,2,1}}   // below, can only go up
		   		     nAvail := 2
	   CASE nx == 10 .and. ny == 5  // Can only choose 2!
	       	aNext := {{9,5,4}, ; // Right, can only go Right
		   		     {9,4,2}}   // below, can only go Down
		   		     nAvail := 2
	   CASE ny == 1  // Can only choose 3!
	  		aNext := {{max(1,nx-1),1,4}, ;  // Left, can only move right
		   			 {min(10,nx+1),1,3}, ; // Right, can only go left
		   			 {nx,2,1}}   // below, can only go up
		   		     nAvail := 3
	   CASE ny == 5  // Can only choose 3!
	  		aNext := {{max(1,nx-1),5,4}, ;  // Left, can only move right
		   			 {min(10,nx+1),5,3}, ; // Right, can only go left
		   			 {nx,4,2}}   // Above, can only go down
		   		     nAvail := 3
	   CASE nx == 1 // Can only choose 3!
	  		aNext := {{1,max(1,ny-1),2}, ;  // Above, can only move down
		   			 {2,ny,3}, ; // Right, can only go left
		   			 {1,min(5,ny+1),1}}   // below, can only go up
		   		     nAvail := 3
	   CASE nx == 10 // Can only choose 3!
	  		aNext := {{10,max(1,ny-1),2}, ;  // Above, can only move down
		   			 {9,ny,4}, ;  // Left, can only move right
		   			 {10,min(5,ny+1),1}}   // below, can only go up
		   		     nAvail := 3
	   OTHERWISE
	  		aNext := {{nx,max(1,ny-1),2}, ;  // Above, can only move down
		   			 {max(1,nx-1),ny,4}, ;  // Left, can only move right
		   			 {min(10,nx+1),ny,3}, ; // Right, can only go left
		   			 {nx,min(5,ny+1),1}}   // below, can only go up
	   				 nAvail := 4
	   ENDCASE

	   // Pick one of the four above
	   nNext := Nurd(nAvail) + 1

	   nx := aNext[nNext][1]
	   ny := aNext[nNext][2]
	   nDir := aNext[nNext][3]
enddo
restscreen(0,0,maxrow(),maxcol(),cSc)
setcursor(anc_cu)
setpos(nrow,ncol)
return .t.

FUNCTION nurd(nX)
seed:=FRAC(seed*9821+.211327)
RETURN INT(seed*nX)

